import { Class, DeepPartial, Filter, QueryService } from '@rezonate/nestjs-query-core';
import { CreateManyInputType, CreateOneInputType, MutationArgsType, SubscriptionArgsType } from '../types';
import { ResolverClass, ServiceResolver, SubscriptionResolverOpts } from './resolver.interface';
export type CreatedEvent<DTO> = {
    [eventName: string]: DTO;
};
export interface CreateResolverOpts<DTO> extends SubscriptionResolverOpts {
    /**
     * The Input DTO that should be used to create records.
     */
    CreateDTOClass?: Class<DeepPartial<DTO>>;
    /**
     * The class to be used for `createOne` input.
     */
    CreateOneInput?: Class<CreateOneInputType<DTO>>;
    /**
     * The class to be used for `createMany` input.
     */
    CreateManyInput?: Class<CreateManyInputType<DTO>>;
    createOneMutationName?: string;
    createManyMutationName?: string;
}
export interface CreateResolver<DTO, QS extends QueryService<DTO>> extends ServiceResolver<DTO, QS> {
    createOne(input: MutationArgsType<CreateOneInputType<DTO>>, authorizeFilter?: Filter<DTO>): Promise<DTO>;
    createMany(input: MutationArgsType<CreateManyInputType<DTO>>, authorizeFilter?: Filter<DTO>): Promise<DTO[]>;
    createdSubscription(input?: SubscriptionArgsType<DTO>, authorizeFilter?: Filter<DTO>): AsyncIterator<CreatedEvent<DTO>>;
}
/**
 * @internal
 * Mixin to add `create` graphql endpoints.
 */
export declare const Creatable: <DTO, QS extends QueryService<DTO>>(DTOClass: Class<DTO>, opts: CreateResolverOpts<DTO>) => <B extends Class<ServiceResolver<DTO, QS>>>(BaseClass: B) => Class<CreateResolver<DTO, QS>> & B;
/**
 * Factory to create a new abstract class that can be extended to add `create` endpoints.
 *
 * Assume we have `TodoItemDTO`, you can create a resolver with `createOneTodoItem` and `createManyTodoItems` graphql
 * query endpoints using the following code.
 *
 * ```ts
 * @Resolver()
 * export class TodoItemResolver extends CreateResolver(TodoItemDTO) {
 *   constructor(readonly service: TodoItemService) {
 *    super(service);
 *   }
 * }
 * ```
 *
 * @param DTOClass - The DTO class that should be returned from the `createOne` and `createMany` endpoint.
 * @param opts - Options to customize endpoints.
 * @typeparam DTO - The type of DTO that should be created.
 * @typeparam C - The create DTO type.
 */
export declare const CreateResolver: <DTO, QS extends QueryService<DTO> = QueryService<DTO>>(DTOClass: Class<DTO>, opts?: CreateResolverOpts<DTO>) => ResolverClass<DTO, QS, CreateResolver<DTO, QS>>;
