"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CRUDResolver = void 0;
const tslib_1 = require("tslib");
const graphql_1 = require("@nestjs/graphql");
const common_1 = require("../common");
const aggregate_by_time_resolver_1 = require("./aggregate.by.time.resolver");
const aggregate_resolver_1 = require("./aggregate.resolver");
const create_resolver_1 = require("./create.resolver");
const delete_resolver_1 = require("./delete.resolver");
const read_resolver_1 = require("./read.resolver");
const reference_resolver_1 = require("./reference.resolver");
const relations_1 = require("./relations");
const resolver_interface_1 = require("./resolver.interface");
const update_resolver_1 = require("./update.resolver");
function extractRelatableOpts(opts) {
    const { enableTotalCount, enableAggregate, aggregate } = opts;
    return (0, common_1.mergeBaseResolverOpts)({ enableAggregate, enableTotalCount, ...aggregate }, opts);
}
function extractAggregateResolverOpts(opts) {
    const { enableAggregate, aggregate } = opts;
    return (0, common_1.mergeBaseResolverOpts)({ enabled: enableAggregate, ...aggregate }, opts);
}
function extractCreateResolverOpts(opts) {
    const { CreateDTOClass, enableSubscriptions, create } = opts;
    return (0, common_1.mergeBaseResolverOpts)({ CreateDTOClass, enableSubscriptions, ...create }, opts);
}
function extractReadResolverOpts(opts) {
    const { enableTotalCount, pagingStrategy, read } = opts;
    return (0, common_1.mergeBaseResolverOpts)({
        enableTotalCount,
        pagingStrategy, ...read,
    }, opts);
}
function extractUpdateResolverOpts(opts) {
    const { UpdateDTOClass, enableSubscriptions, update } = opts;
    return (0, common_1.mergeBaseResolverOpts)({ UpdateDTOClass, enableSubscriptions, ...update }, opts);
}
function extractDeleteResolverOpts(opts) {
    const { enableSubscriptions, delete: deleteArgs } = opts;
    return (0, common_1.mergeBaseResolverOpts)({ enableSubscriptions, ...deleteArgs }, opts);
}
/**
 * Factory to create a resolver that includes all CRUD methods from [[CreateResolver]], [[ReadResolver]],
 * [[UpdateResolver]], and [[DeleteResolver]].
 *
 * ```ts
 * import { CRUDResolver } from '@rezonate/nestjs-query-graphql';
 * import { Resolver } from '@nestjs/graphql';
 * import { TodoItemDTO } from './dto/todo-item.dto';
 * import { TodoItemService } from './todo-item.service';
 *
 * @Resolver()
 * export class TodoItemResolver extends CRUDResolver(TodoItemDTO) {
 *   constructor(readonly service: TodoItemService) {
 *     super(service);
 *   }
 * }
 * ```
 * @param DTOClass - The DTO Class that the resolver is for. All methods will use types derived from this class.
 * @param opts - Options to customize the resolver.
 */
// eslint-disable-next-line @typescript-eslint/no-redeclare -- intentional
const CRUDResolver = (DTOClass, opts = {}) => {
    let BaseResolver = class BaseResolver extends resolver_interface_1.BaseServiceResolver {
    };
    BaseResolver = tslib_1.__decorate([
        (0, graphql_1.Resolver)(() => DTOClass, { isAbstract: true })
    ], BaseResolver);
    const Mixins = [];
    Mixins.push((0, reference_resolver_1.Referenceable)(DTOClass, opts.referenceBy ?? {}));
    Mixins.push((0, relations_1.Relatable)(DTOClass, extractRelatableOpts(opts)));
    if (opts.aggregate?.disabled)
        Mixins.push((0, aggregate_resolver_1.Aggregateable)(DTOClass, extractAggregateResolverOpts(opts)));
    if (opts.aggregate?.disabled)
        Mixins.push((0, aggregate_by_time_resolver_1.AggregateableByTime)(DTOClass, extractAggregateResolverOpts(opts)));
    if (opts.create?.disabled)
        Mixins.push((0, create_resolver_1.Creatable)(DTOClass, extractCreateResolverOpts(opts)));
    if (opts.read?.disabled)
        Mixins.push((0, read_resolver_1.Readable)(DTOClass, extractReadResolverOpts(opts)));
    if (opts.update?.disabled)
        Mixins.push((0, update_resolver_1.Updateable)(DTOClass, extractUpdateResolverOpts(opts)));
    if (opts.delete?.disabled)
        Mixins.push((0, delete_resolver_1.Deletable)(DTOClass, extractDeleteResolverOpts(opts)));
    return Mixins.reduce((CurrResolver, action) => action(CurrResolver), BaseResolver);
};
exports.CRUDResolver = CRUDResolver;
//# sourceMappingURL=crud.resolver.js.map