"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilterType = FilterType;
exports.DeleteFilterType = DeleteFilterType;
exports.UpdateFilterType = UpdateFilterType;
exports.SubscriptionFilterType = SubscriptionFilterType;
exports.AggregateFilterType = AggregateFilterType;
const tslib_1 = require("tslib");
const graphql_1 = require("@nestjs/graphql");
const class_transformer_1 = require("class-transformer");
const class_validator_1 = require("class-validator");
const upper_case_first_1 = require("upper-case-first");
const common_1 = require("../../common");
const decorators_1 = require("../../decorators");
const has_required_filter_1 = require("../../decorators/has-required.filter");
const field_comparison_1 = require("./field-comparison");
const helpers_1 = require("./helpers");
function getObjectTypeName(DTOClass) {
    return (0, common_1.getGraphqlObjectName)(DTOClass, 'No fields found to create FilterType.');
}
function getFilterableRelations(relations) {
    const filterableRelations = {};
    Object.keys(relations).forEach((r) => {
        const opts = relations[r];
        if (opts && opts.allowFiltering) {
            filterableRelations[r] = opts.DTO;
        }
    });
    return filterableRelations;
}
const filterObjCache = new Map();
function handleFilterFields(TClass, name, fields) {
    const GraphQLFilter = filterObjCache.get(name);
    if (!GraphQLFilter) {
        throw new Error(`No filter type created for ${TClass.name}`);
    }
    if (!fields.length) {
        throw new Error(`No fields found to create GraphQLFilter for ${TClass.name}`);
    }
    const { one = {}, many = {} } = (0, decorators_1.getRelations)(TClass);
    const filterableRelations = { ...getFilterableRelations(one), ...getFilterableRelations(many) };
    const proto = GraphQLFilter.prototype;
    const { baseName } = (0, common_1.getDTONames)(TClass);
    fields.forEach(({ propertyName, target, advancedOptions, returnTypeFunc }) => {
        const FC = (0, field_comparison_1.createFilterComparisonType)({
            FieldType: target,
            fieldName: `${baseName}${(0, upper_case_first_1.upperCaseFirst)(propertyName)}`,
            allowedComparisons: advancedOptions?.allowedComparisons,
            isJSON: advancedOptions?.isJSON,
            futureDate: advancedOptions?.futureDate,
            returnTypeFunc,
        });
        const nullable = advancedOptions?.filterRequired !== true;
        (0, class_validator_1.ValidateNested)()(proto, propertyName);
        if (advancedOptions?.filterRequired) {
            (0, has_required_filter_1.HasRequiredFilter)()(proto, propertyName);
        }
        (0, graphql_1.Field)(() => FC, { nullable })(proto, propertyName);
        (0, class_transformer_1.Type)(() => FC)(proto, propertyName);
    });
    Object.entries(filterableRelations).forEach(([field, FieldType]) => {
        // eslint-disable-next-line @typescript-eslint/no-use-before-define
        const FC = getOrCreateFilterType(FieldType);
        (0, class_validator_1.ValidateNested)()(proto, field);
        (0, graphql_1.Field)(() => FC, { nullable: true })(proto, field);
        (0, class_transformer_1.Type)(() => FC)(proto, field);
    });
}
function getOrCreateFilterType(TClass) {
    const name = `${getObjectTypeName(TClass)}Filter`;
    const cache = filterObjCache.get(name);
    if (cache)
        return cache;
    const fields = (0, decorators_1.getFilterableFields)(TClass);
    const isShareable = (0, decorators_1.getShareableDTO)(TClass);
    const hasRequiredFilters = fields.some((f) => f.advancedOptions?.filterRequired === true);
    const { allowedBooleanExpressions, disableFreeTextQuery } = (0, decorators_1.getQueryOptions)(TClass) ?? {};
    const isNotAllowedComparison = (val) => !(0, helpers_1.isInAllowedList)(allowedBooleanExpressions, val);
    let GraphQLFilter = class GraphQLFilter {
        static { this.hasRequiredFilters = hasRequiredFilters; }
    };
    tslib_1.__decorate([
        (0, decorators_1.SkipIf)(() => disableFreeTextQuery, (0, graphql_1.Field)(() => String, { nullable: true })),
        (0, class_transformer_1.Type)(() => String),
        tslib_1.__metadata("design:type", String)
    ], GraphQLFilter.prototype, "freeTextQuery", void 0);
    tslib_1.__decorate([
        (0, class_validator_1.ValidateNested)(),
        (0, decorators_1.SkipIf)(() => isNotAllowedComparison('and'), (0, graphql_1.Field)(() => [GraphQLFilter], { nullable: true })),
        (0, class_transformer_1.Type)(() => GraphQLFilter),
        tslib_1.__metadata("design:type", Array)
    ], GraphQLFilter.prototype, "and", void 0);
    tslib_1.__decorate([
        (0, class_validator_1.ValidateNested)(),
        (0, decorators_1.SkipIf)(() => isNotAllowedComparison('or'), (0, graphql_1.Field)(() => [GraphQLFilter], { nullable: true })),
        (0, class_transformer_1.Type)(() => GraphQLFilter),
        tslib_1.__metadata("design:type", Array)
    ], GraphQLFilter.prototype, "or", void 0);
    GraphQLFilter = tslib_1.__decorate([
        (0, graphql_1.InputType)(name)
    ], GraphQLFilter);
    if (isShareable)
        (0, decorators_1.setShareable)(GraphQLFilter);
    filterObjCache.set(name, GraphQLFilter);
    handleFilterFields(TClass, name, fields);
    return GraphQLFilter;
}
function FilterType(TClass) {
    return getOrCreateFilterType(TClass);
}
function DeleteFilterType(TClass) {
    return getOrCreateFilterType(TClass);
}
function UpdateFilterType(TClass) {
    return getOrCreateFilterType(TClass);
}
function SubscriptionFilterType(TClass) {
    return getOrCreateFilterType(TClass);
}
function AggregateFilterType(TClass) {
    return getOrCreateFilterType(TClass);
}
//# sourceMappingURL=filter.type.js.map